# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from kmip.core import enums
from kmip.core import exceptions
from kmip.core import objects
from kmip.core import primitives
from kmip.core import utils
from kmip.core.messages.payloads import base


class GetAttributeListRequestPayload(base.RequestPayload):
    """
    A request payload for the GetAttributeList operation.

    The payload can contain the ID of the managed object the attributes should
    belong too. If omitted, the server will use the ID placeholder by default.
    See Section 4.13 of the KMIP 1.1 specification for more information.

    Attributes:
        unique_identifier: The unique ID of the managed object with which the
            retrieved attributes should be associated.
    """

    def __init__(self, unique_identifier=None):
        """
        Construct a GetAttributeList request payload.

        Args:
            unique_identifier (string): The ID of the managed object with
                which the retrieved attribute names should be associated.
                Optional, defaults to None.
        """
        super(GetAttributeListRequestPayload, self).__init__()

        self._unique_identifier = None

        self.unique_identifier = unique_identifier

    @property
    def unique_identifier(self):
        if self._unique_identifier:
            return self._unique_identifier.value
        else:
            return self._unique_identifier

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, six.string_types):
            self._unique_identifier = primitives.TextString(
                value=value,
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
        else:
            raise TypeError("Unique identifier must be a string.")

    def read(self, input_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the GetAttributeList request payload and decode
        it into its constituent parts.

        Args:
            input_buffer (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.
        """
        super(GetAttributeListRequestPayload, self).read(
            input_buffer,
            kmip_version=kmip_version
        )
        local_buffer = utils.BytearrayStream(input_buffer.read(self.length))

        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_buffer):
            self._unique_identifier = primitives.TextString(
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
            self._unique_identifier.read(
                local_buffer,
                kmip_version=kmip_version
            )
        else:
            self._unique_identifier = None

        self.is_oversized(local_buffer)

    def write(self, output_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the GetAttributeList request payload to a
        stream.

        Args:
            output_buffer (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.
        """
        local_buffer = utils.BytearrayStream()

        if self._unique_identifier:
            self._unique_identifier.write(
                local_buffer,
                kmip_version=kmip_version
            )

        self.length = local_buffer.length()
        super(GetAttributeListRequestPayload, self).write(
            output_buffer,
            kmip_version=kmip_version
        )
        output_buffer.write(local_buffer.buffer)

    def __repr__(self):
        uid = "unique_identifier={0}".format(self.unique_identifier)
        return "GetAttributeListRequestPayload({0})".format(uid)

    def __str__(self):
        return str({'unique_identifier': self.unique_identifier})

    def __eq__(self, other):
        if isinstance(other, GetAttributeListRequestPayload):
            if self.unique_identifier == other.unique_identifier:
                return True
            else:
                return False
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, GetAttributeListRequestPayload):
            return not self.__eq__(other)
        else:
            return NotImplemented


class GetAttributeListResponsePayload(base.ResponsePayload):
    """
    A response payload for the GetAttributeList operation.

    The payload contains the ID of the managed object with which the
    attributes are associated, along with a list of attribute names
    identifying the types of attributes associated with the aforementioned
    managed object.

    Attributes:
        unique_identifier: The unique ID of the managed object with which the
            retrieved attributes should be associated.
        attribute_names: A list of strings identifying the names of the
            attributes associated with the managed object.
    """

    def __init__(self, unique_identifier=None, attribute_names=None):
        """
        Construct a GetAttributeList response payload.

        Args:
            unique_identifier (string): The ID of the managed object with
                which the retrieved attribute names should be associated.
                Optional, defaults to None.
            attribute_names: A list of strings identifying the names of the
                attributes associated with the managed object. Optional,
                defaults to None.
        """

        super(GetAttributeListResponsePayload, self).__init__()

        self._unique_identifier = None
        self._attribute_names = list()

        self.unique_identifier = unique_identifier
        self.attribute_names = attribute_names

    @property
    def unique_identifier(self):
        if self._unique_identifier:
            return self._unique_identifier.value
        else:
            return self._unique_identifier

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, six.string_types):
            self._unique_identifier = primitives.TextString(
                value=value,
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
        else:
            raise TypeError("Unique identifier must be a string.")

    @property
    def attribute_names(self):
        if self._attribute_names:
            names = list()
            for attribute_name in self._attribute_names:
                names.append(attribute_name.value)
            return names
        else:
            return self._attribute_names

    @attribute_names.setter
    def attribute_names(self, value):
        if value is None:
            self._attribute_names = list()
        elif isinstance(value, list):
            names = list()
            for i in range(len(value)):
                name = value[i]
                if not isinstance(name, six.string_types):
                    raise TypeError(
                        "Attribute names must be a list of strings; "
                        "item {0} has type {1}".format(i + 1, type(name))
                    )
                if name not in names:
                    names.append(name)
            self._attribute_names = list()
            for name in names:
                self._attribute_names.append(
                    primitives.TextString(
                        value=name,
                        tag=enums.Tags.ATTRIBUTE_NAME
                    )
                )
        else:
            raise TypeError("Attribute names must be a list of strings.")

    def read(self, input_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the GetAttributeList response payload and
        decode it into its constituent parts.

        Args:
            input_buffer (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            InvalidKmipEncoding: Raised if the unique identifier or attribute
                names are missing from the encoded payload.
        """
        super(GetAttributeListResponsePayload, self).read(
            input_buffer,
            kmip_version=kmip_version
        )
        local_buffer = utils.BytearrayStream(input_buffer.read(self.length))

        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_buffer):
            self._unique_identifier = primitives.TextString(
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
            self._unique_identifier.read(
                local_buffer,
                kmip_version=kmip_version
            )
        else:
            raise exceptions.InvalidKmipEncoding(
                "The GetAttributeList response payload encoding is missing "
                "the unique identifier."
            )

        names = list()
        if kmip_version < enums.KMIPVersion.KMIP_2_0:
            while self.is_tag_next(enums.Tags.ATTRIBUTE_NAME, local_buffer):
                name = primitives.TextString(tag=enums.Tags.ATTRIBUTE_NAME)
                name.read(local_buffer, kmip_version=kmip_version)
                names.append(name)
            if len(names) == 0:
                raise exceptions.InvalidKmipEncoding(
                    "The GetAttributeList response payload encoding is "
                    "missing the attribute names."
                )
            self._attribute_names = names
        else:
            while self.is_tag_next(
                    enums.Tags.ATTRIBUTE_REFERENCE,
                    local_buffer
            ):
                if self.is_type_next(enums.Types.STRUCTURE, local_buffer):
                    reference = objects.AttributeReference()
                    reference.read(local_buffer, kmip_version=kmip_version)
                    names.append(
                        primitives.TextString(
                            value=reference.attribute_name,
                            tag=enums.Tags.ATTRIBUTE_NAME
                        )
                    )
                elif self.is_type_next(enums.Types.ENUMERATION, local_buffer):
                    reference = primitives.Enumeration(
                        enums.Tags,
                        tag=enums.Tags.ATTRIBUTE_REFERENCE
                    )
                    reference.read(local_buffer, kmip_version=kmip_version)
                    name = enums.convert_attribute_tag_to_name(reference.value)
                    names.append(
                        primitives.TextString(
                            value=name,
                            tag=enums.Tags.ATTRIBUTE_NAME
                        )
                    )
                else:
                    raise exceptions.InvalidKmipEncoding(
                        "The GetAttributeList response payload encoding "
                        "contains an invalid AttributeReference type."
                    )
            self._attribute_names = names

        self.is_oversized(local_buffer)

    def write(self, output_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the GetAttributeList response payload to a
        stream.

        Args:
            output_buffer (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            InvalidField: Raised if the unique identifier or attribute name
                are not defined.
        """
        local_buffer = utils.BytearrayStream()

        if self._unique_identifier:
            self._unique_identifier.write(
                local_buffer,
                kmip_version=kmip_version
            )
        else:
            raise exceptions.InvalidField(
                "The GetAttributeList response payload is missing the unique "
                "identifier field."
            )

        if self._attribute_names:
            if kmip_version < enums.KMIPVersion.KMIP_2_0:
                for attribute_name in self._attribute_names:
                    attribute_name.write(
                        local_buffer,
                        kmip_version=kmip_version
                    )
            else:
                # NOTE (ph) This approach simplifies backwards compatible
                #           issues but limits easy support for Attribute
                #           Reference structures going forward, specifically
                #           limiting the use of VendorIdentification for
                #           custom attributes. If custom attributes need to
                #           be retrieved using the GetAttributeList operation
                #           for KMIP 2.0 applications this code will need to
                #           change.
                for attribute_name in self._attribute_names:
                    t = enums.convert_attribute_name_to_tag(
                        attribute_name.value
                    )
                    e = primitives.Enumeration(
                        enums.Tags,
                        value=t,
                        tag=enums.Tags.ATTRIBUTE_REFERENCE
                    )
                    e.write(local_buffer, kmip_version=kmip_version)

        else:
            raise exceptions.InvalidField(
                "The GetAttributeList response payload is missing the "
                "attribute names field."
            )

        self.length = local_buffer.length()
        super(GetAttributeListResponsePayload, self).write(
            output_buffer,
            kmip_version=kmip_version
        )
        output_buffer.write(local_buffer.buffer)

    def __repr__(self):
        unique_identifier = "unique_identifier={0}".format(
            self.unique_identifier
        )
        attribute_names = "attribute_names={0}".format(self.attribute_names)
        return "GetAttributeListResponsePayload({0}, {1})".format(
            unique_identifier,
            attribute_names
        )

    def __str__(self):
        return str({
            'unique_identifier': self.unique_identifier,
            'attribute_names': self.attribute_names
        })

    def __eq__(self, other):
        if isinstance(other, GetAttributeListResponsePayload):
            if self.unique_identifier == other.unique_identifier:
                if set(self.attribute_names) == set(other.attribute_names):
                    return True
                else:
                    return False
            else:
                return False
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, GetAttributeListResponsePayload):
            return not self.__eq__(other)
        else:
            return NotImplemented



# Copyright (c) 2019 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import enums


class TestOrderedEnum(testtools.TestCase):

    def setUp(self):
        super(TestOrderedEnum, self).setUp()

    def tearDown(self):
        super(TestOrderedEnum, self).tearDown()

    def test_greater_than_or_equal(self):
        self.assertTrue(
            enums.KMIPVersion.KMIP_2_0 >= enums.KMIPVersion.KMIP_1_0
        )
        self.assertFalse(
            enums.KMIPVersion.KMIP_1_0 >= enums.KMIPVersion.KMIP_2_0
        )

        self.assertEquals(
            NotImplemented,
            enums.KMIPVersion.KMIP_2_0.__ge__(enums.WrappingMethod.ENCRYPT)
        )

    def test_greater_than(self):
        self.assertTrue(
            enums.KMIPVersion.KMIP_1_3 > enums.KMIPVersion.KMIP_1_1
        )
        self.assertFalse(
            enums.KMIPVersion.KMIP_1_1 > enums.KMIPVersion.KMIP_1_3
        )

        self.assertEquals(
            NotImplemented,
            enums.KMIPVersion.KMIP_2_0.__gt__(enums.WrappingMethod.ENCRYPT)
        )

    def test_less_than_or_equal(self):
        self.assertTrue(
            enums.KMIPVersion.KMIP_1_3 <= enums.KMIPVersion.KMIP_1_4
        )
        self.assertFalse(
            enums.KMIPVersion.KMIP_1_4 <= enums.KMIPVersion.KMIP_1_3
        )

        self.assertEquals(
            NotImplemented,
            enums.KMIPVersion.KMIP_2_0.__le__(enums.WrappingMethod.ENCRYPT)
        )

    def test_less_than(self):
        self.assertTrue(
            enums.KMIPVersion.KMIP_1_3 < enums.KMIPVersion.KMIP_2_0
        )
        self.assertFalse(
            enums.KMIPVersion.KMIP_2_0 < enums.KMIPVersion.KMIP_1_3
        )

        self.assertEquals(
            NotImplemented,
            enums.KMIPVersion.KMIP_2_0.__lt__(enums.WrappingMethod.ENCRYPT)
        )


class TestEnumUtilityFunctions(testtools.TestCase):

    def setUp(self):
        super(TestEnumUtilityFunctions, self).setUp()

    def tearDown(self):
        super(TestEnumUtilityFunctions, self).tearDown()

    def test_get_bit_mask_from_enumerations(self):
        self.assertEqual(
            7,
            enums.get_bit_mask_from_enumerations(
                [
                    enums.StorageStatusMask.ARCHIVAL_STORAGE,
                    enums.StorageStatusMask.DESTROYED_STORAGE,
                    enums.StorageStatusMask.ONLINE_STORAGE
                ]
            )
        )

    def test_get_enumerations_from_bit_mask(self):
        expected = [
                enums.StorageStatusMask.ARCHIVAL_STORAGE,
                enums.StorageStatusMask.DESTROYED_STORAGE,
                enums.StorageStatusMask.ONLINE_STORAGE
        ]
        observed = enums.get_enumerations_from_bit_mask(
            enums.StorageStatusMask,
            7
        )

        self.assertEqual(len(expected), len(observed))
        for x in expected:
            self.assertIn(x, observed)

    def test_is_bit_mask(self):
        self.assertTrue(
            enums.is_bit_mask(
                enums.StorageStatusMask,
                enums.StorageStatusMask.ARCHIVAL_STORAGE.value |
                enums.StorageStatusMask.ONLINE_STORAGE.value
            )
        )

        self.assertFalse(
            enums.is_bit_mask(
                enums.StorageStatusMask,
                enums.StorageStatusMask.DESTROYED_STORAGE
            )
        )

        self.assertFalse(
            enums.is_bit_mask(
                enums.WrappingMethod,
                enums.WrappingMethod.ENCRYPT.value
            )
        )

        self.assertFalse(
            enums.is_bit_mask(
                enums.ProtectionStorageMask,
                0x80000000
            )
        )

    def test_is_enum_value(self):
        result = enums.is_enum_value(
            enums.CryptographicAlgorithm,
            enums.CryptographicAlgorithm.AES
        )
        self.assertTrue(result)

        result = enums.is_enum_value(
            enums.WrappingMethod,
            'invalid'
        )
        self.assertFalse(result)

    def test_convert_attribute_name_to_tag(self):
        self.assertEqual(
            enums.Tags.OBJECT_TYPE,
            enums.convert_attribute_name_to_tag("Object Type")
        )

        args = (enums.Tags.COMMON_ATTRIBUTES, )
        self.assertRaisesRegex(
            ValueError,
            "The attribute name must be a string.",
            enums.convert_attribute_name_to_tag,
            *args
        )

        args = ("invalid", )
        self.assertRaisesRegex(
            ValueError,
            "Unrecognized attribute name: 'invalid'".format(args[0]),
            enums.convert_attribute_name_to_tag,
            *args
        )

    def test_convert_attribute_tag_to_name(self):
        self.assertEqual(
            "Always Sensitive",
            enums.convert_attribute_tag_to_name(enums.Tags.ALWAYS_SENSITIVE)
        )

        args = ("invalid", )
        self.assertRaisesRegex(
            ValueError,
            "The attribute tag must be a Tags enumeration.",
            enums.convert_attribute_tag_to_name,
            *args
        )

        args = (enums.Tags.COMMON_ATTRIBUTES, )
        self.assertRaisesRegex(
            ValueError,
            "Unrecognized attribute tag: {}".format(args[0]),
            enums.convert_attribute_tag_to_name,
            *args
        )

    def test_is_attribute(self):
        # Test an attribute introduced in KMIP 1.0
        result = enums.is_attribute(enums.Tags.UNIQUE_IDENTIFIER)
        self.assertTrue(result)

        # Test an attribute introduced in KMIP 1.1
        result = enums.is_attribute(enums.Tags.FRESH)
        self.assertTrue(result)

        # Test an attribute introduced in KMIP 1.2
        result = enums.is_attribute(enums.Tags.KEY_VALUE_PRESENT)
        self.assertTrue(result)

        # Test an attribute introduced in KMIP 1.3
        result = enums.is_attribute(enums.Tags.RANDOM_NUMBER_GENERATOR)
        self.assertTrue(result)

        # Test an attribute introduced in KMIP 1.4
        result = enums.is_attribute(enums.Tags.COMMENT)
        self.assertTrue(result)

        # Test an attribute introduced in KMIP 2.0
        result = enums.is_attribute(enums.Tags.QUANTUM_SAFE)
        self.assertTrue(result)

    def test_is_attribute_added_in_kmip_1_0(self):
        result = enums.is_attribute(
            enums.Tags.UNIQUE_IDENTIFIER,
            enums.KMIPVersion.KMIP_1_0
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.UNIQUE_IDENTIFIER,
            enums.KMIPVersion.KMIP_1_1
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.UNIQUE_IDENTIFIER,
            enums.KMIPVersion.KMIP_1_2
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.UNIQUE_IDENTIFIER,
            enums.KMIPVersion.KMIP_1_3
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.UNIQUE_IDENTIFIER,
            enums.KMIPVersion.KMIP_1_4
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.UNIQUE_IDENTIFIER,
            enums.KMIPVersion.KMIP_2_0
        )
        self.assertTrue(result)

    def test_is_attribute_added_in_kmip_1_1(self):
        result = enums.is_attribute(
            enums.Tags.FRESH,
            enums.KMIPVersion.KMIP_1_0
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.FRESH,
            enums.KMIPVersion.KMIP_1_1
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.FRESH,
            enums.KMIPVersion.KMIP_1_2
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.FRESH,
            enums.KMIPVersion.KMIP_1_3
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.FRESH,
            enums.KMIPVersion.KMIP_1_4
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.FRESH,
            enums.KMIPVersion.KMIP_2_0
        )
        self.assertTrue(result)

    def test_is_attribute_added_in_kmip_1_2(self):
        result = enums.is_attribute(
            enums.Tags.KEY_VALUE_PRESENT,
            enums.KMIPVersion.KMIP_1_0
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.KEY_VALUE_PRESENT,
            enums.KMIPVersion.KMIP_1_1
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.KEY_VALUE_PRESENT,
            enums.KMIPVersion.KMIP_1_2
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.KEY_VALUE_PRESENT,
            enums.KMIPVersion.KMIP_1_3
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.KEY_VALUE_PRESENT,
            enums.KMIPVersion.KMIP_1_4
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.KEY_VALUE_PRESENT,
            enums.KMIPVersion.KMIP_2_0
        )
        self.assertTrue(result)

    def test_is_attribute_added_in_kmip_1_3(self):
        result = enums.is_attribute(
            enums.Tags.RANDOM_NUMBER_GENERATOR,
            enums.KMIPVersion.KMIP_1_0
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.RANDOM_NUMBER_GENERATOR,
            enums.KMIPVersion.KMIP_1_1
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.RANDOM_NUMBER_GENERATOR,
            enums.KMIPVersion.KMIP_1_2
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.RANDOM_NUMBER_GENERATOR,
            enums.KMIPVersion.KMIP_1_3
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.RANDOM_NUMBER_GENERATOR,
            enums.KMIPVersion.KMIP_1_4
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.RANDOM_NUMBER_GENERATOR,
            enums.KMIPVersion.KMIP_2_0
        )
        self.assertTrue(result)

    def test_is_attribute_added_in_kmip_1_4(self):
        result = enums.is_attribute(
            enums.Tags.COMMENT,
            enums.KMIPVersion.KMIP_1_0
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.COMMENT,
            enums.KMIPVersion.KMIP_1_1
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.COMMENT,
            enums.KMIPVersion.KMIP_1_2
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.COMMENT,
            enums.KMIPVersion.KMIP_1_3
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.COMMENT,
            enums.KMIPVersion.KMIP_1_4
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.COMMENT,
            enums.KMIPVersion.KMIP_2_0
        )
        self.assertTrue(result)

    def test_is_attribute_added_in_kmip_2_0(self):
        result = enums.is_attribute(
            enums.Tags.QUANTUM_SAFE,
            enums.KMIPVersion.KMIP_1_0
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.QUANTUM_SAFE,
            enums.KMIPVersion.KMIP_1_1
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.QUANTUM_SAFE,
            enums.KMIPVersion.KMIP_1_2
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.QUANTUM_SAFE,
            enums.KMIPVersion.KMIP_1_3
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.QUANTUM_SAFE,
            enums.KMIPVersion.KMIP_1_4
        )
        self.assertFalse(result)

        result = enums.is_attribute(
            enums.Tags.QUANTUM_SAFE,
            enums.KMIPVersion.KMIP_2_0
        )
        self.assertTrue(result)

    def test_is_attribute_removed_in_kmip_2_0(self):
        result = enums.is_attribute(
            enums.Tags.CUSTOM_ATTRIBUTE,
            enums.KMIPVersion.KMIP_1_0
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.CUSTOM_ATTRIBUTE,
            enums.KMIPVersion.KMIP_1_1
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.CUSTOM_ATTRIBUTE,
            enums.KMIPVersion.KMIP_1_2
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.CUSTOM_ATTRIBUTE,
            enums.KMIPVersion.KMIP_1_3
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.CUSTOM_ATTRIBUTE,
            enums.KMIPVersion.KMIP_1_4
        )
        self.assertTrue(result)

        result = enums.is_attribute(
            enums.Tags.CUSTOM_ATTRIBUTE,
            enums.KMIPVersion.KMIP_2_0
        )
        self.assertFalse(result)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import sys

from kmip.core import enums
from kmip.demos import utils

from kmip.pie import client
from kmip.pie import objects


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    parser = utils.build_cli_parser()
    opts, args = parser.parse_args(sys.argv[1:])

    config = opts.config

    value = (
        b'\x30\x82\x03\x12\x30\x82\x01\xFA\xA0\x03\x02\x01\x02\x02\x01\x01'
        b'\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05\x00\x30'
        b'\x3B\x31\x0B\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D'
        b'\x30\x0B\x06\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30'
        b'\x0C\x06\x03\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30'
        b'\x0B\x06\x03\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x1E\x17\x0D'
        b'\x31\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x17\x0D\x32'
        b'\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x30\x3B\x31\x0B'
        b'\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D\x30\x0B\x06'
        b'\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30\x0C\x06\x03'
        b'\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30\x0B\x06\x03'
        b'\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x82\x01\x22\x30\x0D\x06'
        b'\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01\x05\x00\x03\x82\x01\x0F'
        b'\x00\x30\x82\x01\x0A\x02\x82\x01\x01\x00\xAB\x7F\x16\x1C\x00\x42'
        b'\x49\x6C\xCD\x6C\x6D\x4D\xAD\xB9\x19\x97\x34\x35\x35\x77\x76\x00'
        b'\x3A\xCF\x54\xB7\xAF\x1E\x44\x0A\xFB\x80\xB6\x4A\x87\x55\xF8\x00'
        b'\x2C\xFE\xBA\x6B\x18\x45\x40\xA2\xD6\x60\x86\xD7\x46\x48\x34\x6D'
        b'\x75\xB8\xD7\x18\x12\xB2\x05\x38\x7C\x0F\x65\x83\xBC\x4D\x7D\xC7'
        b'\xEC\x11\x4F\x3B\x17\x6B\x79\x57\xC4\x22\xE7\xD0\x3F\xC6\x26\x7F'
        b'\xA2\xA6\xF8\x9B\x9B\xEE\x9E\x60\xA1\xD7\xC2\xD8\x33\xE5\xA5\xF4'
        b'\xBB\x0B\x14\x34\xF4\xE7\x95\xA4\x11\x00\xF8\xAA\x21\x49\x00\xDF'
        b'\x8B\x65\x08\x9F\x98\x13\x5B\x1C\x67\xB7\x01\x67\x5A\xBD\xBC\x7D'
        b'\x57\x21\xAA\xC9\xD1\x4A\x7F\x08\x1F\xCE\xC8\x0B\x64\xE8\xA0\xEC'
        b'\xC8\x29\x53\x53\xC7\x95\x32\x8A\xBF\x70\xE1\xB4\x2E\x7B\xB8\xB7'
        b'\xF4\xE8\xAC\x8C\x81\x0C\xDB\x66\xE3\xD2\x11\x26\xEB\xA8\xDA\x7D'
        b'\x0C\xA3\x41\x42\xCB\x76\xF9\x1F\x01\x3D\xA8\x09\xE9\xC1\xB7\xAE'
        b'\x64\xC5\x41\x30\xFB\xC2\x1D\x80\xE9\xC2\xCB\x06\xC5\xC8\xD7\xCC'
        b'\xE8\x94\x6A\x9A\xC9\x9B\x1C\x28\x15\xC3\x61\x2A\x29\xA8\x2D\x73'
        b'\xA1\xF9\x93\x74\xFE\x30\xE5\x49\x51\x66\x2A\x6E\xDA\x29\xC6\xFC'
        b'\x41\x13\x35\xD5\xDC\x74\x26\xB0\xF6\x05\x02\x03\x01\x00\x01\xA3'
        b'\x21\x30\x1F\x30\x1D\x06\x03\x55\x1D\x0E\x04\x16\x04\x14\x04\xE5'
        b'\x7B\xD2\xC4\x31\xB2\xE8\x16\xE1\x80\xA1\x98\x23\xFA\xC8\x58\x27'
        b'\x3F\x6B\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05'
        b'\x00\x03\x82\x01\x01\x00\xA8\x76\xAD\xBC\x6C\x8E\x0F\xF0\x17\x21'
        b'\x6E\x19\x5F\xEA\x76\xBF\xF6\x1A\x56\x7C\x9A\x13\xDC\x50\xD1\x3F'
        b'\xEC\x12\xA4\x27\x3C\x44\x15\x47\xCF\xAB\xCB\x5D\x61\xD9\x91\xE9'
        b'\x66\x31\x9D\xF7\x2C\x0D\x41\xBA\x82\x6A\x45\x11\x2F\xF2\x60\x89'
        b'\xA2\x34\x4F\x4D\x71\xCF\x7C\x92\x1B\x4B\xDF\xAE\xF1\x60\x0D\x1B'
        b'\xAA\xA1\x53\x36\x05\x7E\x01\x4B\x8B\x49\x6D\x4F\xAE\x9E\x8A\x6C'
        b'\x1D\xA9\xAE\xB6\xCB\xC9\x60\xCB\xF2\xFA\xE7\x7F\x58\x7E\xC4\xBB'
        b'\x28\x20\x45\x33\x88\x45\xB8\x8D\xD9\xAE\xEA\x53\xE4\x82\xA3\x6E'
        b'\x73\x4E\x4F\x5F\x03\xB9\xD0\xDF\xC4\xCA\xFC\x6B\xB3\x4E\xA9\x05'
        b'\x3E\x52\xBD\x60\x9E\xE0\x1E\x86\xD9\xB0\x9F\xB5\x11\x20\xC1\x98'
        b'\x34\xA9\x97\xB0\x9C\xE0\x8D\x79\xE8\x13\x11\x76\x2F\x97\x4B\xB1'
        b'\xC8\xC0\x91\x86\xC4\xD7\x89\x33\xE0\xDB\x38\xE9\x05\x08\x48\x77'
        b'\xE1\x47\xC7\x8A\xF5\x2F\xAE\x07\x19\x2F\xF1\x66\xD1\x9F\xA9\x4A'
        b'\x11\xCC\x11\xB2\x7E\xD0\x50\xF7\xA2\x7F\xAE\x13\xB2\x05\xA5\x74'
        b'\xC4\xEE\x00\xAA\x8B\xD6\x5D\x0D\x70\x57\xC9\x85\xC8\x39\xEF\x33'
        b'\x6A\x44\x1E\xD5\x3A\x53\xC6\xB6\xB6\x96\xF1\xBD\xEB\x5F\x7E\xA8'
        b'\x11\xEB\xB2\x5A\x7F\x86')
    usage_mask = [
        enums.CryptographicUsageMask.ENCRYPT,
        enums.CryptographicUsageMask.VERIFY]
    name = 'Demo X.509 Certificate'

    cert = objects.X509Certificate(value, usage_mask, name)
    cert.operation_policy_name = opts.operation_policy_name

    # Build the client and connect to the server
    with client.ProxyKmipClient(
            config=config,
            config_file=opts.config_file
    ) as client:
        try:
            uid = client.register(cert)
            logger.info("Successfully registered certificate with ID: "
                        "{0}".format(uid))
        except Exception as e:
            logger.error(e)



# Copyright (c) 2017 Pure Storage, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import attributes
from kmip.core import enums
from kmip.core import exceptions
from kmip.core.enums import Tags
from kmip.core.messages.payloads import base
from kmip.core.objects import Data, MACData
from kmip.core.utils import BytearrayStream


# 4.33
class MACRequestPayload(base.RequestPayload):

    def __init__(self,
                 unique_identifier=None,
                 cryptographic_parameters=None,
                 data=None):

        super(MACRequestPayload, self).__init__()

        self._unique_identifier = None
        self._cryptographic_parameters = None
        self._data = None

        self.unique_identifier = unique_identifier
        self.cryptographic_parameters = cryptographic_parameters
        self.data = data

    @property
    def unique_identifier(self):
        return self._unique_identifier

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, attributes.UniqueIdentifier):
            self._unique_identifier = value
        else:
            raise TypeError("unique identifier must be UniqueIdentifier type")

    @property
    def cryptographic_parameters(self):
        return self._cryptographic_parameters

    @cryptographic_parameters.setter
    def cryptographic_parameters(self, value):
        if value is None:
            self._cryptographic_parameters = None
        elif isinstance(value, attributes.CryptographicParameters):
            self._cryptographic_parameters = value
        else:
            raise TypeError("cryptographic parameters must "
                            "be CryptographicParameters type")

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, value):
        if value is None:
            self._data = None
        elif isinstance(value, Data):
            self._data = value
        else:
            raise TypeError("data must be Data type")

    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        super(MACRequestPayload, self).read(
            istream,
            kmip_version=kmip_version
        )
        tstream = BytearrayStream(istream.read(self.length))

        if self.is_tag_next(Tags.UNIQUE_IDENTIFIER, tstream):
            self.unique_identifier = attributes.UniqueIdentifier()
            self.unique_identifier.read(tstream, kmip_version=kmip_version)

        if self.is_tag_next(Tags.CRYPTOGRAPHIC_PARAMETERS, tstream):
            self.cryptographic_parameters = \
                attributes.CryptographicParameters()
            self.cryptographic_parameters.read(
                tstream,
                kmip_version=kmip_version
            )

        if self.is_tag_next(Tags.DATA, tstream):
            self.data = Data()
            self.data.read(tstream, kmip_version=kmip_version)
        else:
            raise exceptions.InvalidKmipEncoding(
                "expected mac request data not found"
            )

        self.is_oversized(tstream)

    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        tstream = BytearrayStream()

        if self._unique_identifier is not None:
            self._unique_identifier.write(tstream, kmip_version=kmip_version)
        if self._cryptographic_parameters is not None:
            self._cryptographic_parameters.write(
                tstream,
                kmip_version=kmip_version
            )
        if self._data is not None:
            self.data.write(tstream, kmip_version=kmip_version)
        else:
            raise exceptions.InvalidField(
                "The mac request data is required"
            )

        self.length = tstream.length()
        super(MACRequestPayload, self).write(
            ostream,
            kmip_version=kmip_version
        )
        ostream.write(tstream.buffer)


class MACResponsePayload(base.ResponsePayload):

    def __init__(self,
                 unique_identifier=None,
                 mac_data=None):
        super(MACResponsePayload, self).__init__()

        self._unique_identifier = None
        self._mac_data = None

        self.unique_identifier = unique_identifier
        self.mac_data = mac_data

    @property
    def unique_identifier(self):
        return self._unique_identifier

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, attributes.UniqueIdentifier):
            self._unique_identifier = value
        else:
            raise TypeError("unique identifier must be UniqueIdentifier type")

    @property
    def mac_data(self):
        return self._mac_data

    @mac_data.setter
    def mac_data(self, value):
        if value is None:
            self._mac_data = None
        elif isinstance(value, MACData):
            self._mac_data = value
        else:
            raise TypeError("mac_data must be MACData type")

    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        super(MACResponsePayload, self).read(
            istream,
            kmip_version=kmip_version
        )
        tstream = BytearrayStream(istream.read(self.length))

        if self.is_tag_next(Tags.UNIQUE_IDENTIFIER, tstream):
            self._unique_identifier = attributes.UniqueIdentifier()
            self._unique_identifier.read(tstream, kmip_version=kmip_version)
        else:
            raise exceptions.InvalidKmipEncoding(
                "expected mac response unique identifier not found"
            )

        if self.is_tag_next(Tags.MAC_DATA, tstream):
            self._mac_data = MACData()
            self._mac_data.read(tstream, kmip_version=kmip_version)
        else:
            raise exceptions.InvalidKmipEncoding(
                "expected mac response mac data not found"
            )

        self.is_oversized(tstream)

    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        tstream = BytearrayStream()

        if self._unique_identifier is not None:
            self._unique_identifier.write(tstream, kmip_version=kmip_version)
        else:
            raise exceptions.InvalidField(
                "The mac response unique identifier is required"
            )
        if self._mac_data is not None:
            self._mac_data.write(tstream, kmip_version=kmip_version)
        else:
            raise exceptions.InvalidField(
                "The mac response mac data is required"
            )
        self.length = tstream.length()
        super(MACResponsePayload, self).write(
            ostream,
            kmip_version=kmip_version
        )
        ostream.write(tstream.buffer)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from kmip.core import enums
from kmip.core import objects
from kmip.core import primitives
from kmip.core import secrets
from kmip.core import utils
from kmip.core.factories import secrets as secret_factory
from kmip.core.messages.payloads import base


class GetRequestPayload(base.RequestPayload):
    """
    A request payload for the Get operation.

    Attributes:
        unique_identifier: The unique ID of the managed object to retrieve
            from the server.
        key_format_type: The format of the returned object, if it is a key.
        key_compression_type: The compression method to be used for the
            returned object, if it is an elliptic curve public key.
        key_wrapping_specification: A collection of settings specifying how
            the returned object should be cryptographically wrapped if it is
            a key.
    """

    def __init__(self,
                 unique_identifier=None,
                 key_format_type=None,
                 key_compression_type=None,
                 key_wrapping_specification=None):
        """
        Construct a Get request payload struct.

        Args:
            unique_identifier (string): The ID of the managed object (e.g., a
                symmetric key) to retrieve. Optional, defaults to None.
            key_format_type (KeyFormatType): A KeyFormatType enumeration that
                specifies the format in which the object should be returned.
                Optional, defaults to None.
            key_compression_type (KeyCompressionType): A KeyCompressionType
                enumeration that specifies the compression method to be used
                when returning elliptic curve public keys. Optional, defaults
                to None.
            key_wrapping_specification (KeyWrappingSpecification): A
                KeyWrappingSpecification struct that specifies keys and other
                information for wrapping the returned object. Optional,
                defaults to None.
        """
        super(GetRequestPayload, self).__init__()

        self._unique_identifier = None
        self._key_format_type = None
        self._key_compression_type = None
        self._key_wrapping_specification = None

        self.unique_identifier = unique_identifier
        self.key_format_type = key_format_type
        self.key_compression_type = key_compression_type
        self.key_wrapping_specification = key_wrapping_specification

    @property
    def unique_identifier(self):
        if self._unique_identifier:
            return self._unique_identifier.value
        else:
            return None

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, six.string_types):
            self._unique_identifier = primitives.TextString(
                value=value,
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
        else:
            raise TypeError("Unique identifier must be a string.")

    @property
    def key_format_type(self):
        if self._key_format_type:
            return self._key_format_type.value
        else:
            return None

    @key_format_type.setter
    def key_format_type(self, value):
        if value is None:
            self._key_format_type = None
        elif isinstance(value, enums.KeyFormatType):
            self._key_format_type = primitives.Enumeration(
                enums.KeyFormatType,
                value=value,
                tag=enums.Tags.KEY_FORMAT_TYPE
            )
        else:
            raise TypeError(
                "Key format type must be a KeyFormatType enumeration."
            )

    @property
    def key_compression_type(self):
        if self._key_compression_type:
            return self._key_compression_type.value
        else:
            return None

    @key_compression_type.setter
    def key_compression_type(self, value):
        if value is None:
            self._key_compression_type = None
        elif isinstance(value, enums.KeyCompressionType):
            self._key_compression_type = primitives.Enumeration(
                enums.KeyCompressionType,
                value=value,
                tag=enums.Tags.KEY_COMPRESSION_TYPE
            )
        else:
            raise TypeError(
                "Key compression type must be a KeyCompressionType "
                "enumeration."
            )

    @property
    def key_wrapping_specification(self):
        return self._key_wrapping_specification

    @key_wrapping_specification.setter
    def key_wrapping_specification(self, value):
        if value is None:
            self._key_wrapping_specification = None
        elif isinstance(value, objects.KeyWrappingSpecification):
            self._key_wrapping_specification = value
        else:
            raise TypeError(
                "Key wrapping specification must be a "
                "KeyWrappingSpecification struct."
            )

    def read(self, input_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the Get request payload and decode it into its
        constituent parts.

        Args:
            input_stream (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.
        """
        super(GetRequestPayload, self).read(
            input_stream,
            kmip_version=kmip_version
        )
        local_stream = utils.BytearrayStream(input_stream.read(self.length))

        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_stream):
            self._unique_identifier = primitives.TextString(
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
            self._unique_identifier.read(
                local_stream,
                kmip_version=kmip_version
            )

        if self.is_tag_next(enums.Tags.KEY_FORMAT_TYPE, local_stream):
            self._key_format_type = primitives.Enumeration(
                enum=enums.KeyFormatType,
                tag=enums.Tags.KEY_FORMAT_TYPE
            )
            self._key_format_type.read(
                local_stream,
                kmip_version=kmip_version
            )

        if self.is_tag_next(enums.Tags.KEY_COMPRESSION_TYPE, local_stream):
            self._key_compression_type = primitives.Enumeration(
                enum=enums.KeyCompressionType,
                tag=enums.Tags.KEY_COMPRESSION_TYPE
            )
            self._key_compression_type.read(
                local_stream,
                kmip_version=kmip_version
            )

        if self.is_tag_next(
                enums.Tags.KEY_WRAPPING_SPECIFICATION,
                local_stream
        ):
            self._key_wrapping_specification = \
                objects.KeyWrappingSpecification()
            self._key_wrapping_specification.read(
                local_stream,
                kmip_version=kmip_version
            )

        self.is_oversized(local_stream)

    def write(self, output_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the Get request payload to a stream.

        Args:
            output_stream (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.
        """
        local_stream = utils.BytearrayStream()

        if self._unique_identifier is not None:
            self._unique_identifier.write(
                local_stream,
                kmip_version=kmip_version
            )
        if self._key_format_type is not None:
            self._key_format_type.write(
                local_stream,
                kmip_version=kmip_version
            )
        if self._key_compression_type is not None:
            self._key_compression_type.write(
                local_stream,
                kmip_version=kmip_version
            )
        if self._key_wrapping_specification is not None:
            self._key_wrapping_specification.write(
                local_stream,
                kmip_version=kmip_version
            )

        self.length = local_stream.length()
        super(GetRequestPayload, self).write(
            output_stream,
            kmip_version=kmip_version
        )
        output_stream.write(local_stream.buffer)

    def __eq__(self, other):
        if isinstance(other, GetRequestPayload):
            if self.unique_identifier != other.unique_identifier:
                return False
            elif self.key_format_type != other.key_format_type:
                return False
            elif self.key_compression_type != other.key_compression_type:
                return False
            elif self.key_wrapping_specification != \
                    other.key_wrapping_specification:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, GetRequestPayload):
            return not (self == other)
        else:
            return NotImplemented

    def __repr__(self):
        args = ", ".join([
            "unique_identifier='{0}'".format(self.unique_identifier),
            "key_format_type={0}".format(self.key_format_type),
            "key_compression_type={0}".format(self.key_compression_type),
            "key_wrapping_specification={0}".format(
                repr(self.key_wrapping_specification)
            )
        ])
        return "GetRequestPayload({0})".format(args)

    def __str__(self):
        return str({
            'unique_identifier': self.unique_identifier,
            'key_format_type': self.key_format_type,
            'key_compression_type': self.key_compression_type,
            'key_wrapping_specification': self.key_wrapping_specification
        })


class GetResponsePayload(base.ResponsePayload):
    """
    A response payload for the Get operation.

    Attributes:
        object_type: The type of the managed object being returned.
        unique_identifier: The unique ID of the managed object being returned.
        secret: The managed object being returned.
    """

    def __init__(self,
                 object_type=None,
                 unique_identifier=None,
                 secret=None):
        """
        Construct a Get response payload struct.

        Args:
            object_type (ObjectType): An ObjectType enumeration specifying the
                type of managed object being returned. Optional, defaults to
                None. Required for read/write.
            unique_identifier (string): The ID of the managed object (e.g., a
                symmetric key) being returned. Optional, defaults to None.
                Required for read/write.
            secret (various): The managed object struct being returned. Must
                be one of the following:

                Optional, defaults to None. Required for read/write.
        """
        super(GetResponsePayload, self).__init__()

        self._object_type = None
        self._unique_identifier = None
        self._secret = None

        self.object_type = object_type
        self.unique_identifier = unique_identifier
        self.secret = secret

        self.secret_factory = secret_factory.SecretFactory()

    @property
    def object_type(self):
        if self._object_type:
            return self._object_type.value
        else:
            return None

    @object_type.setter
    def object_type(self, value):
        if value is None:
            self._object_type = None
        elif isinstance(value, enums.ObjectType):
            self._object_type = primitives.Enumeration(
                enums.ObjectType,
                value=value,
                tag=enums.Tags.OBJECT_TYPE
            )
        else:
            raise TypeError("Object type must be an ObjectType enumeration.")

    @property
    def unique_identifier(self):
        if self._unique_identifier:
            return self._unique_identifier.value
        else:
            return None

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, six.string_types):
            self._unique_identifier = primitives.TextString(
                value=value,
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
        else:
            raise TypeError("Unique identifier must be a string.")

    @property
    def secret(self):
        return self._secret

    @secret.setter
    def secret(self, value):
        if value is None:
            self._secret = None
        elif isinstance(
                value,
                (
                    secrets.Certificate,
                    secrets.OpaqueObject,
                    secrets.PrivateKey,
                    secrets.PublicKey,
                    secrets.SecretData,
                    secrets.SplitKey,
                    secrets.SymmetricKey,
                    secrets.Template
                )
        ):
            self._secret = value
        else:
            raise TypeError(
                "Secret must be one of the following structs: Certificate, "
                "OpaqueObject, PrivateKey, PublicKey, SecretData, SplitKey, "
                "SymmetricKey, Template"
            )

    def read(self, input_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the Get response payload and decode it
        into its constituent parts.

        Args:
            input_stream (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            ValueError: Raised if the object type, unique identifier, or
                secret attributes are missing from the encoded payload.
        """
        super(GetResponsePayload, self).read(
            input_stream,
            kmip_version=kmip_version
        )
        local_stream = utils.BytearrayStream(input_stream.read(self.length))

        if self.is_tag_next(enums.Tags.OBJECT_TYPE, local_stream):
            self._object_type = primitives.Enumeration(
                enum=enums.ObjectType,
                tag=enums.Tags.OBJECT_TYPE
            )
            self._object_type.read(local_stream, kmip_version=kmip_version)
        else:
            raise ValueError(
                "Parsed payload encoding is missing the object type field."
            )

        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_stream):
            self._unique_identifier = primitives.TextString(
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
            self._unique_identifier.read(
                local_stream,
                kmip_version=kmip_version
            )
        else:
            raise ValueError(
                "Parsed payload encoding is missing the unique identifier "
                "field."
            )

        self.secret = self.secret_factory.create(self.object_type)
        if self.is_tag_next(self._secret.tag, local_stream):
            self._secret.read(local_stream, kmip_version=kmip_version)
        else:
            raise ValueError(
                "Parsed payload encoding is missing the secret field."
            )

        self.is_oversized(local_stream)

    def write(self, output_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the Get response payload to a stream.

        Args:
            output_stream (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            ValueError: Raised if the object type, unique identifier, or
                secret attributes are missing from the payload struct.
        """
        local_stream = utils.BytearrayStream()

        if self.object_type:
            self._object_type.write(local_stream, kmip_version=kmip_version)
        else:
            raise ValueError("Payload is missing the object type field.")

        if self.unique_identifier:
            self._unique_identifier.write(
                local_stream,
                kmip_version=kmip_version
            )
        else:
            raise ValueError(
                "Payload is missing the unique identifier field."
            )

        if self.secret:
            self._secret.write(local_stream, kmip_version=kmip_version)
        else:
            raise ValueError("Payload is missing the secret field.")

        self.length = local_stream.length()
        super(GetResponsePayload, self).write(
            output_stream,
            kmip_version=kmip_version
        )
        output_stream.write(local_stream.buffer)

    def __eq__(self, other):
        if isinstance(other, GetResponsePayload):
            if self.object_type != other.object_type:
                return False
            elif self.unique_identifier != other.unique_identifier:
                return False
            elif self.secret != other.secret:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, GetResponsePayload):
            return not (self == other)
        else:
            return NotImplemented

    def __repr__(self):
        args = ", ".join([
            "object_type={0}".format(self.object_type),
            "unique_identifier='{0}'".format(self.unique_identifier),
            "secret={0}".format(repr(self.secret))
        ])
        return "GetResponsePayload({0})".format(args)

    def __str__(self):
        return str({
            'object_type': self.object_type,
            'unique_identifier': self.unique_identifier,
            'secret': self.secret
        })



# Copyright (c) 2019 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import sys

from kmip.core import enums
from kmip.demos import utils
from kmip.pie import client


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    # Build and parse arguments
    parser = utils.build_cli_parser(enums.Operation.GET_ATTRIBUTES)
    opts, args = parser.parse_args(sys.argv[1:])

    config = opts.config
    uid = opts.uuid

    # Exit early if the UUID is not specified
    if uid is None:
        logger.error('No ID provided, exiting early from demo')
        sys.exit()

    # Build the client and connect to the server
    with client.ProxyKmipClient(
            config=config,
            config_file=opts.config_file
    ) as client:
        try:
            _, attributes = client.get_attributes(
                uid, attribute_names=opts.attribute_names)
            logger.info("Successfully retrieved {0} attributes:".format(
                len(attributes)))
            for attribute in attributes:
                logger.info("Attribute {0}: {1}".format(
                    attribute.attribute_name, attribute.attribute_value))
        except Exception as e:
            logger.error(e)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import enums
from kmip.core.messages import contents
from kmip.core import utils


class TestProtocolVersion(testtools.TestCase):

    def setUp(self):
        super(TestProtocolVersion, self).setUp()

        # Encoding obtained from the KMIP 1.1 testing document, Section 3.1.1.
        #
        # This encoding matches the following set of values:
        # ProtocolVersion
        #     ProtocolVersionMajor - 1
        #     ProtocolVersionMinor - 1

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x69\x01\x00\x00\x00\x20'
            b'\x42\x00\x6A\x02\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x6B\x02\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
        )

        self.encoding_no_major_number = utils.BytearrayStream(
            b'\x42\x00\x69\x01\x00\x00\x00\x10'
            b'\x42\x00\x6B\x02\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
        )

        self.encoding_no_minor_number = utils.BytearrayStream(
            b'\x42\x00\x69\x01\x00\x00\x00\x10'
            b'\x42\x00\x6A\x02\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestProtocolVersion, self).tearDown()

    def test_init(self):
        """
        Test that a ProtocolVersion struct can be constructed with no
        arguments.
        """
        struct = contents.ProtocolVersion()

        self.assertEqual(None, struct.major)
        self.assertEqual(None, struct.minor)

    def test_init_with_args(self):
        """
        Test that a ProtocolVersion struct can be constructed with valid
        values.
        """
        struct = contents.ProtocolVersion(1, 1)

        self.assertEqual(1, struct.major)
        self.assertEqual(1, struct.minor)

    def test_invalid_protocol_version_major(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the major protocol version number of a ProtocolVersion struct.
        """
        struct = contents.ProtocolVersion()
        args = (struct, 'major', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Major protocol version number must be an integer.",
            setattr,
            *args
        )

    def test_invalid_protocol_version_minor(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the minor protocol version number of a ProtocolVersion struct.
        """
        struct = contents.ProtocolVersion()
        args = (struct, 'minor', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Minor protocol version number must be an integer.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a ProtocolVersion struct can be read from a data stream.
        """
        struct = contents.ProtocolVersion()

        self.assertEqual(None, struct.major)
        self.assertEqual(None, struct.minor)

        struct.read(self.full_encoding)

        self.assertEqual(1, struct.major)
        self.assertEqual(1, struct.minor)

    def test_read_missing_major_number(self):
        """
        Test that a ValueError gets raised when a required ProtocolVersion
        struct attribute is missing from the struct encoding.
        """
        struct = contents.ProtocolVersion()
        args = (self.encoding_no_major_number, )
        self.assertRaisesRegex(
            ValueError,
            "Invalid encoding missing the major protocol version number.",
            struct.read,
            *args
        )

    def test_read_missing_minor_number(self):
        """
        Test that a ValueError gets raised when a required ProtocolVersion
        struct attribute is missing from the struct encoding.
        """
        struct = contents.ProtocolVersion()
        args = (self.encoding_no_minor_number, )
        self.assertRaisesRegex(
            ValueError,
            "Invalid encoding missing the minor protocol version number.",
            struct.read,
            *args
        )

    def test_write(self):
        """
        Test that a ProtocolVersion struct can be written to a data stream.
        """
        struct = contents.ProtocolVersion(1, 1)
        stream = utils.BytearrayStream()
        struct.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_missing_major_number(self):
        """
        Test that a ValueError gets raised when a required ProtocolVersion
        struct attribute is missing when encoding the struct.
        """
        struct = contents.ProtocolVersion(None, 1)
        stream = utils.BytearrayStream()
        args = (stream, )
        self.assertRaisesRegex(
            ValueError,
            "Invalid struct missing the major protocol version number.",
            struct.write,
            *args
        )

    def test_write_missing_minor_number(self):
        """
        Test that a ValueError gets raised when a required ProtocolVersion
        struct attribute is missing when encoding the struct.
        """
        struct = contents.ProtocolVersion(1, None)
        stream = utils.BytearrayStream()
        args = (stream, )
        self.assertRaisesRegex(
            ValueError,
            "Invalid struct missing the minor protocol version number.",
            struct.write,
            *args
        )

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        ProtocolVersion structs with the same data.
        """
        a = contents.ProtocolVersion(1, 0)
        b = contents.ProtocolVersion(1, 0)

        self.assertTrue(a == b)

    def test_equal_on_not_equal(self):
        """
        Test that the equality operator returns False when comparing two
        ProtocolVersion structs with different data.
        """
        a = contents.ProtocolVersion(1, 0)
        b = contents.ProtocolVersion(0, 1)

        self.assertFalse(a == b)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        ProtocolVersion structs with different types.
        """
        a = contents.ProtocolVersion(1, 0)
        b = "invalid"

        self.assertFalse(a == b)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        ProtocolVersion structs with the same data.
        """
        a = contents.ProtocolVersion(1, 0)
        b = contents.ProtocolVersion(1, 0)

        self.assertFalse(a != b)

    def test_not_equal_on_not_equal(self):
        """
        Test that the inequality operator returns True when comparing two
        ProtocolVersion structs with different data.
        """
        a = contents.ProtocolVersion(1, 0)
        b = contents.ProtocolVersion(0, 1)

        self.assertTrue(a != b)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        ProtocolVersion structs with different types.
        """
        a = contents.ProtocolVersion(1, 0)
        b = "invalid"

        self.assertTrue(a != b)

    def test_less_than(self):
        """
        Test that the less than operator correctly returns True/False when
        comparing two different ProtocolVersions.
        """
        a = contents.ProtocolVersion(1, 0)
        b = contents.ProtocolVersion(1, 1)
        c = contents.ProtocolVersion(2, 0)
        d = contents.ProtocolVersion(0, 2)

        self.assertTrue(a < b)
        self.assertFalse(b < a)
        self.assertFalse(a < a)
        self.assertTrue(a < c)
        self.assertFalse(c < a)
        self.assertFalse(c < d)
        self.assertTrue(d < c)

        # A direct call to __lt__ is required here due to differences in how
        # Python 2 and Python 3 treat comparison operators.
        self.assertEqual(NotImplemented, a.__lt__('invalid'))

    def test_greater_than(self):
        """
        Test that the greater than operator correctly returns True/False when
        comparing two different ProtocolVersions.
        """
        a = contents.ProtocolVersion(1, 0)
        b = contents.ProtocolVersion(1, 1)
        c = contents.ProtocolVersion(2, 0)
        d = contents.ProtocolVersion(0, 2)

        self.assertFalse(a > b)
        self.assertTrue(b > a)
        self.assertFalse(a > a)
        self.assertFalse(a > c)
        self.assertTrue(c > a)
        self.assertTrue(c > d)
        self.assertFalse(d > c)

        # A direct call to __gt__ is required here due to differences in how
        # Python 2 and Python 3 treat comparison operators.
        self.assertEqual(NotImplemented, a.__gt__('invalid'))

    def test_less_than_or_equal(self):
        """
        Test that the less than or equal operator correctly returns True/False
        when comparing two different ProtocolVersions.
        """
        a = contents.ProtocolVersion(1, 0)
        b = contents.ProtocolVersion(1, 1)
        c = contents.ProtocolVersion(2, 0)
        d = contents.ProtocolVersion(0, 2)

        self.assertTrue(a <= b)
        self.assertFalse(b <= a)
        self.assertTrue(a <= a)
        self.assertTrue(a <= c)
        self.assertFalse(c <= a)
        self.assertFalse(c <= d)
        self.assertTrue(d <= c)

        # A direct call to __le__ is required here due to differences in how
        # Python 2 and Python 3 treat comparison operators.
        self.assertEqual(NotImplemented, a.__le__('invalid'))

    def test_greater_than_or_equal(self):
        """
        Test that the greater than or equal operator correctly returns
        True/False when comparing two different ProtocolVersions.
        """
        a = contents.ProtocolVersion(1, 0)
        b = contents.ProtocolVersion(1, 1)
        c = contents.ProtocolVersion(2, 0)
        d = contents.ProtocolVersion(0, 2)

        self.assertFalse(a >= b)
        self.assertTrue(b >= a)
        self.assertTrue(a >= a)
        self.assertFalse(a >= c)
        self.assertTrue(c >= a)
        self.assertTrue(c >= d)
        self.assertFalse(d >= c)

        # A direct call to __ge__ is required here due to differences in how
        # Python 2 and Python 3 treat comparison operators.
        self.assertEqual(NotImplemented, a.__ge__('invalid'))

    def test_repr(self):
        """
        Test that repr can be applied to a ProtocolVersion struct.
        """
        struct = contents.ProtocolVersion(1, 0)

        self.assertEqual(
            "ProtocolVersion(major=1, minor=0)",
            "{}".format(repr(struct))
        )

    def test_str(self):
        """
        Test that str can be applied to a ProtocolVersion struct.
        """
        struct = contents.ProtocolVersion(1, 0)

        self.assertEqual("1.0", str(struct))


class TestContents(testtools.TestCase):

    def setUp(self):
        super(TestContents, self).setUp()

    def tearDown(self):
        super(TestContents, self).tearDown()

    def test_protocol_version_to_kmip_version_kmip_1_0(self):
        """
        Test the conversion from ProtocolVersion(1, 0) to KMIPVersion.KMIP_1_0.
        """
        result = contents.protocol_version_to_kmip_version(
            contents.ProtocolVersion(1, 0)
        )
        self.assertEqual(result, enums.KMIPVersion.KMIP_1_0)

    def test_protocol_version_to_kmip_version_kmip_1_1(self):
        """
        Test the conversion from ProtocolVersion(1, 1) to KMIPVersion.KMIP_1_1.
        """
        result = contents.protocol_version_to_kmip_version(
            contents.ProtocolVersion(1, 1)
        )
        self.assertEqual(result, enums.KMIPVersion.KMIP_1_1)

    def test_protocol_version_to_kmip_version_kmip_1_2(self):
        """
        Test the conversion from ProtocolVersion(1, 2) to KMIPVersion.KMIP_1_2.
        """
        result = contents.protocol_version_to_kmip_version(
            contents.ProtocolVersion(1, 2)
        )
        self.assertEqual(result, enums.KMIPVersion.KMIP_1_2)

    def test_protocol_version_to_kmip_version_kmip_1_3(self):
        """
        Test the conversion from ProtocolVersion(1, 3) to KMIPVersion.KMIP_1_3.
        """
        result = contents.protocol_version_to_kmip_version(
            contents.ProtocolVersion(1, 3)
        )
        self.assertEqual(result, enums.KMIPVersion.KMIP_1_3)

    def test_protocol_version_to_kmip_version_kmip_1_4(self):
        """
        Test the conversion from ProtocolVersion(1, 4) to KMIPVersion.KMIP_1_4.
        """
        result = contents.protocol_version_to_kmip_version(
            contents.ProtocolVersion(1, 4)
        )
        self.assertEqual(result, enums.KMIPVersion.KMIP_1_4)

    def test_protocol_version_to_kmip_version_invalid_minor(self):
        """
        Test the conversion from invalid ProtocolVersion minor value to None.
        """
        result = contents.protocol_version_to_kmip_version(
            contents.ProtocolVersion(1, 5)
        )
        self.assertIsNone(result)

    def test_protocol_version_to_kmip_version_invalid_major(self):
        """
        Test the conversion from invalid ProtocolVersion major value to None.
        """
        result = contents.protocol_version_to_kmip_version(
            contents.ProtocolVersion(9, 5)
        )
        self.assertIsNone(result)



# Copyright (c) 2019 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import enums
from kmip.core import exceptions
from kmip.core import objects
from kmip.core import primitives
from kmip.core import secrets
from kmip.core import utils

from kmip.core.messages import payloads


class TestRegisterRequestPayload(testtools.TestCase):

    def setUp(self):
        super(TestRegisterRequestPayload, self).setUp()

        self.certificate_value = (
            b'\x30\x82\x03\x12\x30\x82\x01\xFA\xA0\x03\x02\x01\x02\x02\x01\x01'
            b'\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05\x00\x30'
            b'\x3B\x31\x0B\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D'
            b'\x30\x0B\x06\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30'
            b'\x0C\x06\x03\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30'
            b'\x0B\x06\x03\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x1E\x17\x0D'
            b'\x31\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x17\x0D\x32'
            b'\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x30\x3B\x31\x0B'
            b'\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D\x30\x0B\x06'
            b'\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30\x0C\x06\x03'
            b'\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30\x0B\x06\x03'
            b'\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x82\x01\x22\x30\x0D\x06'
            b'\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01\x05\x00\x03\x82\x01\x0F'
            b'\x00\x30\x82\x01\x0A\x02\x82\x01\x01\x00\xAB\x7F\x16\x1C\x00\x42'
            b'\x49\x6C\xCD\x6C\x6D\x4D\xAD\xB9\x19\x97\x34\x35\x35\x77\x76\x00'
            b'\x3A\xCF\x54\xB7\xAF\x1E\x44\x0A\xFB\x80\xB6\x4A\x87\x55\xF8\x00'
            b'\x2C\xFE\xBA\x6B\x18\x45\x40\xA2\xD6\x60\x86\xD7\x46\x48\x34\x6D'
            b'\x75\xB8\xD7\x18\x12\xB2\x05\x38\x7C\x0F\x65\x83\xBC\x4D\x7D\xC7'
            b'\xEC\x11\x4F\x3B\x17\x6B\x79\x57\xC4\x22\xE7\xD0\x3F\xC6\x26\x7F'
            b'\xA2\xA6\xF8\x9B\x9B\xEE\x9E\x60\xA1\xD7\xC2\xD8\x33\xE5\xA5\xF4'
            b'\xBB\x0B\x14\x34\xF4\xE7\x95\xA4\x11\x00\xF8\xAA\x21\x49\x00\xDF'
            b'\x8B\x65\x08\x9F\x98\x13\x5B\x1C\x67\xB7\x01\x67\x5A\xBD\xBC\x7D'
            b'\x57\x21\xAA\xC9\xD1\x4A\x7F\x08\x1F\xCE\xC8\x0B\x64\xE8\xA0\xEC'
            b'\xC8\x29\x53\x53\xC7\x95\x32\x8A\xBF\x70\xE1\xB4\x2E\x7B\xB8\xB7'
            b'\xF4\xE8\xAC\x8C\x81\x0C\xDB\x66\xE3\xD2\x11\x26\xEB\xA8\xDA\x7D'
            b'\x0C\xA3\x41\x42\xCB\x76\xF9\x1F\x01\x3D\xA8\x09\xE9\xC1\xB7\xAE'
            b'\x64\xC5\x41\x30\xFB\xC2\x1D\x80\xE9\xC2\xCB\x06\xC5\xC8\xD7\xCC'
            b'\xE8\x94\x6A\x9A\xC9\x9B\x1C\x28\x15\xC3\x61\x2A\x29\xA8\x2D\x73'
            b'\xA1\xF9\x93\x74\xFE\x30\xE5\x49\x51\x66\x2A\x6E\xDA\x29\xC6\xFC'
            b'\x41\x13\x35\xD5\xDC\x74\x26\xB0\xF6\x05\x02\x03\x01\x00\x01\xA3'
            b'\x21\x30\x1F\x30\x1D\x06\x03\x55\x1D\x0E\x04\x16\x04\x14\x04\xE5'
            b'\x7B\xD2\xC4\x31\xB2\xE8\x16\xE1\x80\xA1\x98\x23\xFA\xC8\x58\x27'
            b'\x3F\x6B\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05'
            b'\x00\x03\x82\x01\x01\x00\xA8\x76\xAD\xBC\x6C\x8E\x0F\xF0\x17\x21'
            b'\x6E\x19\x5F\xEA\x76\xBF\xF6\x1A\x56\x7C\x9A\x13\xDC\x50\xD1\x3F'
            b'\xEC\x12\xA4\x27\x3C\x44\x15\x47\xCF\xAB\xCB\x5D\x61\xD9\x91\xE9'
            b'\x66\x31\x9D\xF7\x2C\x0D\x41\xBA\x82\x6A\x45\x11\x2F\xF2\x60\x89'
            b'\xA2\x34\x4F\x4D\x71\xCF\x7C\x92\x1B\x4B\xDF\xAE\xF1\x60\x0D\x1B'
            b'\xAA\xA1\x53\x36\x05\x7E\x01\x4B\x8B\x49\x6D\x4F\xAE\x9E\x8A\x6C'
            b'\x1D\xA9\xAE\xB6\xCB\xC9\x60\xCB\xF2\xFA\xE7\x7F\x58\x7E\xC4\xBB'
            b'\x28\x20\x45\x33\x88\x45\xB8\x8D\xD9\xAE\xEA\x53\xE4\x82\xA3\x6E'
            b'\x73\x4E\x4F\x5F\x03\xB9\xD0\xDF\xC4\xCA\xFC\x6B\xB3\x4E\xA9\x05'
            b'\x3E\x52\xBD\x60\x9E\xE0\x1E\x86\xD9\xB0\x9F\xB5\x11\x20\xC1\x98'
            b'\x34\xA9\x97\xB0\x9C\xE0\x8D\x79\xE8\x13\x11\x76\x2F\x97\x4B\xB1'
            b'\xC8\xC0\x91\x86\xC4\xD7\x89\x33\xE0\xDB\x38\xE9\x05\x08\x48\x77'
            b'\xE1\x47\xC7\x8A\xF5\x2F\xAE\x07\x19\x2F\xF1\x66\xD1\x9F\xA9\x4A'
            b'\x11\xCC\x11\xB2\x7E\xD0\x50\xF7\xA2\x7F\xAE\x13\xB2\x05\xA5\x74'
            b'\xC4\xEE\x00\xAA\x8B\xD6\x5D\x0D\x70\x57\xC9\x85\xC8\x39\xEF\x33'
            b'\x6A\x44\x1E\xD5\x3A\x53\xC6\xB6\xB6\x96\xF1\xBD\xEB\x5F\x7E\xA8'
            b'\x11\xEB\xB2\x5A\x7F\x86'
        )

        # Encoding obtained from the KMIP 1.1 testing document, Section 13.2.2.
        # Modified to exclude the Link attribute.
        #
        # TODO (ph) Add the Link attribute back in once Links are supported.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Object Type - Certificate
        #     Template Attribute
        #         Attribute
        #             Attribute Name - Cryptographic Usage Mask
        #             Attribute Value - Sign | Verify
        #     Certificate
        #         Certificate Type - X.509
        #         Certificate Value -
        #             0x30820312308201FAA003020102020101300D06092A864886F70D01
        #             01050500303B310B3009060355040613025553310D300B060355040A
        #             130454455354310E300C060355040B13054F41534953310D300B0603
        #             55040313044B4D4950301E170D3130313130313233353935395A170D
        #             3230313130313233353935395A303B310B3009060355040613025553
        #             310D300B060355040A130454455354310E300C060355040B13054F41
        #             534953310D300B060355040313044B4D495030820122300D06092A86
        #             4886F70D01010105000382010F003082010A0282010100AB7F161C00
        #             42496CCD6C6D4DADB919973435357776003ACF54B7AF1E440AFB80B6
        #             4A8755F8002CFEBA6B184540A2D66086D74648346D75B8D71812B205
        #             387C0F6583BC4D7DC7EC114F3B176B7957C422E7D03FC6267FA2A6F8
        #             9B9BEE9E60A1D7C2D833E5A5F4BB0B1434F4E795A41100F8AA214900
        #             DF8B65089F98135B1C67B701675ABDBC7D5721AAC9D14A7F081FCEC8
        #             0B64E8A0ECC8295353C795328ABF70E1B42E7BB8B7F4E8AC8C810CDB
        #             66E3D21126EBA8DA7D0CA34142CB76F91F013DA809E9C1B7AE64C541
        #             30FBC21D80E9C2CB06C5C8D7CCE8946A9AC99B1C2815C3612A29A82D
        #             73A1F99374FE30E54951662A6EDA29C6FC411335D5DC7426B0F60502
        #             03010001A321301F301D0603551D0E0416041404E57BD2C431B2E816
        #             E180A19823FAC858273F6B300D06092A864886F70D01010505000382
        #             010100A876ADBC6C8E0FF017216E195FEA76BFF61A567C9A13DC50D1
        #             3FEC12A4273C441547CFABCB5D61D991E966319DF72C0D41BA826A45
        #             112FF26089A2344F4D71CF7C921B4BDFAEF1600D1BAAA15336057E01
        #             4B8B496D4FAE9E8A6C1DA9AEB6CBC960CBF2FAE77F587EC4BB282045
        #             338845B88DD9AEEA53E482A36E734E4F5F03B9D0DFC4CAFC6BB34EA9
        #             053E52BD609EE01E86D9B09FB51120C19834A997B09CE08D79E81311
        #             762F974BB1C8C09186C4D78933E0DB38E905084877E147C78AF52FAE
        #             07192FF166D19FA94A11CC11B27ED050F7A27FAE13B205A574C4EE00
        #             AA8BD65D0D7057C985C839EF336A441ED53A53C6B6B696F1BDEB5F7E
        #             A811EBB25A7F86
        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x03\x88'
            b'\x42\x00\x57\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x91\x01\x00\x00\x00\x38'
            b'\x42\x00\x08\x01\x00\x00\x00\x30'
            b'\x42\x00\x0A\x07\x00\x00\x00\x18'
            b'\x43\x72\x79\x70\x74\x6F\x67\x72\x61\x70\x68\x69\x63\x20\x55\x73'
            b'\x61\x67\x65\x20\x4D\x61\x73\x6B'
            b'\x42\x00\x0B\x02\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\x13\x01\x00\x00\x03\x30'
            b'\x42\x00\x1D\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x1E\x08\x00\x00\x03\x16' + self.certificate_value +
            b'\x00\x00'
        )

        # Encoding obtained from the KMIP 1.1 testing document, Section 13.2.2.
        # Modified to exclude the Link attribute. Manually converted into the
        # KMIP 2.0 format.
        #
        # TODO (ph) Add the Link attribute back in once Links are supported.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Object Type - Certificate
        #     Attributes
        #         Cryptographic Usage Mask - Sign | Verify
        #     Certificate
        #         Certificate Type - X.509
        #         Certificate Value - See comment for the full encoding.
        #     Protection Storage Masks
        #         Protection Storage Mask - Software | Hardware
        self.full_encoding_with_attributes = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x03\x78'
            b'\x42\x00\x57\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x01\x25\x01\x00\x00\x00\x10'
            b'\x42\x00\x2C\x02\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\x13\x01\x00\x00\x03\x30'
            b'\x42\x00\x1D\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x1E\x08\x00\x00\x03\x16' + self.certificate_value +
            b'\x00\x00'
            b'\x42\x01\x5F\x01\x00\x00\x00\x10'
            b'\x42\x01\x5E\x02\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
        )

        # Encoding obtained from the KMIP 1.1 testing document, Section 13.2.2.
        # Modified to exclude the Link attribute.
        #
        # TODO (ph) Add the Link attribute back in once Links are supported.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Template Attribute
        #         Attribute
        #             Attribute Name - Cryptographic Usage Mask
        #             Attribute Value - Sign | Verify
        #     Certificate
        #         Certificate Type - X.509
        #         Certificate Value - See comment for the full encoding.
        self.no_object_type_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x03\x78'
            b'\x42\x00\x91\x01\x00\x00\x00\x38'
            b'\x42\x00\x08\x01\x00\x00\x00\x30'
            b'\x42\x00\x0A\x07\x00\x00\x00\x18'
            b'\x43\x72\x79\x70\x74\x6F\x67\x72\x61\x70\x68\x69\x63\x20\x55\x73'
            b'\x61\x67\x65\x20\x4D\x61\x73\x6B'
            b'\x42\x00\x0B\x02\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\x13\x01\x00\x00\x03\x30'
            b'\x42\x00\x1D\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x1E\x08\x00\x00\x03\x16' + self.certificate_value +
            b'\x00\x00'
        )

        # Encoding obtained from the KMIP 1.1 testing document, Section 13.2.2.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Object Type - Certificate
        #     Certificate
        #         Certificate Type - X.509
        #         Certificate Value - See comment for the full encoding.
        self.no_template_attribute_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x03\x48'
            b'\x42\x00\x57\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x13\x01\x00\x00\x03\x30'
            b'\x42\x00\x1D\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x1E\x08\x00\x00\x03\x16' + self.certificate_value +
            b'\x00\x00'
        )

        # Encoding obtained from the KMIP 1.1 testing document, Section 13.2.2.
        # Modified to exclude the Link attribute.
        #
        # TODO (ph) Add the Link attribute back in once Links are supported.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Object Type - Certificate
        #     Template Attribute
        #         Attribute
        #             Attribute Name - Cryptographic Usage Mask
        #             Attribute Value - Sign | Verify
        self.no_managed_object_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x50'
            b'\x42\x00\x57\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x91\x01\x00\x00\x00\x38'
            b'\x42\x00\x08\x01\x00\x00\x00\x30'
            b'\x42\x00\x0A\x07\x00\x00\x00\x18'
            b'\x43\x72\x79\x70\x74\x6F\x67\x72\x61\x70\x68\x69\x63\x20\x55\x73'
            b'\x61\x67\x65\x20\x4D\x61\x73\x6B'
            b'\x42\x00\x0B\x02\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestRegisterRequestPayload, self).tearDown()

    def test_invalid_object_type(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the object type of a Register request payload.
        """
        kwargs = {'object_type': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Object type must be an ObjectType enumeration.",
            payloads.RegisterRequestPayload,
            **kwargs
        )

        args = (
            payloads.RegisterRequestPayload(),
            'object_type',
            'invalid'
        )
        self.assertRaisesRegex(
            TypeError,
            "Object type must be an ObjectType enumeration.",
            setattr,
            *args
        )

    def test_invalid_template_attribute(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the template attribute of a Register request payload.
        """
        kwargs = {'template_attribute': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Template attribute must be a TemplateAttribute structure.",
            payloads.RegisterRequestPayload,
            **kwargs
        )

        args = (
            payloads.RegisterRequestPayload(),
            'template_attribute',
            'invalid'
        )
        self.assertRaisesRegex(
            TypeError,
            "Template attribute must be a TemplateAttribute structure.",
            setattr,
            *args
        )

    def test_invalid_managed_object(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the managed object of a Register request payload.
        """
        kwargs = {'managed_object': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Managed object must be a supported managed object structure.",
            payloads.RegisterRequestPayload,
            **kwargs
        )

        args = (
            payloads.RegisterRequestPayload(),
            'managed_object',
            'invalid'
        )
        self.assertRaisesRegex(
            TypeError,
            "Managed object must be a supported managed object structure.",
            setattr,
            *args
        )

    def test_invalid_protection_storage_masks(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the protection storage masks of a Register request payload.
        """
        kwargs = {"protection_storage_masks": "invalid"}
        self.assertRaisesRegex(
            TypeError,
            "The protection storage masks must be a ProtectionStorageMasks "
            "structure.",
            payloads.RegisterRequestPayload,
            **kwargs
        )
        kwargs = {
            "protection_storage_masks": objects.ProtectionStorageMasks(
                tag=enums.Tags.COMMON_PROTECTION_STORAGE_MASKS
            )
        }
        self.assertRaisesRegex(
            TypeError,
            "The protection storage masks must be a ProtectionStorageMasks "
            "structure with a ProtectionStorageMasks tag.",
            payloads.RegisterRequestPayload,
            **kwargs
        )

        args = (
            payloads.RegisterRequestPayload(),
            "protection_storage_masks",
            "invalid"
        )
        self.assertRaisesRegex(
            TypeError,
            "The protection storage masks must be a ProtectionStorageMasks "
            "structure.",
            setattr,
            *args
        )
        args = (
            payloads.RegisterRequestPayload(),
            "protection_storage_masks",
            objects.ProtectionStorageMasks(
                tag=enums.Tags.COMMON_PROTECTION_STORAGE_MASKS
            )
        )
        self.assertRaisesRegex(
            TypeError,
            "The protection storage masks must be a ProtectionStorageMasks "
            "structure with a ProtectionStorageMasks tag.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a Register request payload can be read from a data stream.
        """
        payload = payloads.RegisterRequestPayload()

        self.assertIsNone(payload.object_type)
        self.assertIsNone(payload.template_attribute)
        self.assertIsNone(payload.managed_object)
        self.assertIsNone(payload.protection_storage_masks)

        payload.read(self.full_encoding)

        self.assertEqual(enums.ObjectType.CERTIFICATE, payload.object_type)
        self.assertEqual(
            objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            payload.template_attribute
        )
        self.assertEqual(
            secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            ),
            payload.managed_object
        )
        self.assertIsNone(payload.protection_storage_masks)

    def test_read_kmip_2_0(self):
        """
        Test that a Register request payload can be read from a data stream
        encoded with the KMIP 2.0 format.
        """
        payload = payloads.RegisterRequestPayload()

        self.assertIsNone(payload.object_type)
        self.assertIsNone(payload.template_attribute)
        self.assertIsNone(payload.managed_object)
        self.assertIsNone(payload.protection_storage_masks)

        payload.read(
            self.full_encoding_with_attributes,
            kmip_version=enums.KMIPVersion.KMIP_2_0
        )

        self.assertEqual(enums.ObjectType.CERTIFICATE, payload.object_type)
        self.assertEqual(
            objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            payload.template_attribute
        )
        self.assertEqual(
            secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            ),
            payload.managed_object
        )
        self.assertEqual(
            objects.ProtectionStorageMasks(protection_storage_masks=[3]),
            payload.protection_storage_masks
        )

    def test_read_missing_object_type(self):
        """
        Test that an InvalidKmipEncoding error is raised during the decoding
        of a Register request payload when the object type is missing from the
        encoding.
        """
        payload = payloads.RegisterRequestPayload()

        args = (self.no_object_type_encoding, )
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The Register request payload encoding is missing the object "
            "type.",
            payload.read,
            *args
        )

    def test_read_missing_template_attribute(self):
        """
        Test that an InvalidKmipEncoding error is raised during the decoding
        of a Register request payload when the template attribute is missing
        from the encoding.
        """
        payload = payloads.RegisterRequestPayload()

        args = (self.no_template_attribute_encoding, )
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The Register request payload encoding is missing the template "
            "attribute.",
            payload.read,
            *args
        )

    def test_read_missing_attributes(self):
        """
        Test that an InvalidKmipEncoding error is raised during the decoding
        of a Register request payload when the attributes structure is missing
        from the encoding.
        """
        payload = payloads.RegisterRequestPayload()

        args = (self.no_template_attribute_encoding, )
        kwargs = {"kmip_version": enums.KMIPVersion.KMIP_2_0}
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The Register request payload encoding is missing the attributes "
            "structure.",
            payload.read,
            *args,
            **kwargs
        )

    def test_read_missing_managed_object(self):
        """
        Test that an InvalidKmipEncoding error is raised during the decoding
        of a Register request payload when the managed object is missing from
        the encoding.
        """
        payload = payloads.RegisterRequestPayload()

        args = (self.no_managed_object_encoding, )
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The Register request payload encoding is missing the managed "
            "object.",
            payload.read,
            *args
        )

    def test_write(self):
        """
        Test that a Register request payload can be written to a data stream.
        """
        payload = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.CERTIFICATE,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            )
        )

        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_kmip_2_0(self):
        """
        Test that a Register request payload can be written to a data stream
        encoded with the KMIP 2.0 format.
        """
        payload = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.CERTIFICATE,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            ),
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )

        stream = utils.BytearrayStream()
        payload.write(stream, kmip_version=enums.KMIPVersion.KMIP_2_0)

        self.assertEqual(len(self.full_encoding_with_attributes), len(stream))
        self.assertEqual(str(self.full_encoding_with_attributes), str(stream))

    def test_write_missing_object_type(self):
        """
        Test that an InvalidField error is raised during the encoding of a
        Register request payload when the payload is missing the object type.
        """
        payload = payloads.RegisterRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            )
        )

        args = (utils.BytearrayStream(), )
        self.assertRaisesRegex(
            exceptions.InvalidField,
            "The Register request payload is missing the object type field.",
            payload.write,
            *args
        )

    def test_write_missing_template_attribute(self):
        """
        Test that an InvalidField error is raised during the encoding of a
        Register request payload when the payload is missing the template
        attribute.
        """
        payload = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.CERTIFICATE,
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            )
        )

        args = (utils.BytearrayStream(), )
        self.assertRaisesRegex(
            exceptions.InvalidField,
            "The Register request payload is missing the template attribute "
            "field.",
            payload.write,
            *args
        )

    def test_write_missing_attributes(self):
        """
        Test that an InvalidField error is raised during the encoding of a
        Register request payload when the payload is missing the attributes
        structure.
        """
        payload = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.CERTIFICATE,
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            )
        )

        args = (utils.BytearrayStream(), )
        kwargs = {"kmip_version": enums.KMIPVersion.KMIP_2_0}
        self.assertRaisesRegex(
            exceptions.InvalidField,
            "The Register request payload is missing the template attribute "
            "field.",
            payload.write,
            *args,
            **kwargs
        )

    def test_write_missing_managed_object(self):
        """
        Test that an InvalidField error is raised during the encoding of a
        Register request payload when the payload is missing the managed
        object.
        """
        payload = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.SECRET_DATA,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            )
        )

        args = (utils.BytearrayStream(), )
        self.assertRaisesRegex(
            exceptions.InvalidField,
            "The Register request payload is missing the managed object "
            "field.",
            payload.write,
            *args
        )

    def test_repr(self):
        """
        Test that repr can be applied to a Register request payload structure.
        """
        payload = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.SECRET_DATA,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.SecretData(
                secret_data_type=primitives.Enumeration(
                    enums.SecretDataType,
                    value=enums.SecretDataType.PASSWORD,
                    tag=enums.Tags.SECRET_DATA_TYPE
                ),
                key_block=objects.KeyBlock(
                    key_format_type=objects.KeyFormatType(
                        enums.KeyFormatType.OPAQUE
                    ),
                    key_value=objects.KeyValue(
                        key_material=objects.KeyMaterial(
                            (
                                b'\x53\x65\x63\x72\x65\x74\x50\x61\x73\x73\x77'
                                b'\x6F\x72\x64'
                            )
                        )
                    )
                )
            ),
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )
        self.assertEqual(
            "RegisterRequestPayload("
            "object_type=ObjectType.SECRET_DATA, "
            "template_attribute=Struct(), "
            "managed_object=Struct(), "
            "protection_storage_masks=ProtectionStorageMasks("
            "protection_storage_masks=[3]))",
            repr(payload)
        )

    def test_str(self):
        """
        Test that str can be applied to a Register request payload structure.
        """
        payload = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.SECRET_DATA,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.SecretData(
                secret_data_type=primitives.Enumeration(
                    enums.SecretDataType,
                    value=enums.SecretDataType.PASSWORD,
                    tag=enums.Tags.SECRET_DATA_TYPE
                ),
                key_block=objects.KeyBlock(
                    key_format_type=objects.KeyFormatType(
                        enums.KeyFormatType.OPAQUE
                    ),
                    key_value=objects.KeyValue(
                        key_material=objects.KeyMaterial(
                            (
                                b'\x53\x65\x63\x72\x65\x74\x50\x61\x73\x73\x77'
                                b'\x6F\x72\x64'
                            )
                        )
                    )
                )
            ),
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )
        self.assertEqual(
            '{'
            '"object_type": ObjectType.SECRET_DATA, '
            '"template_attribute": Struct(), '
            '"managed_object": Struct(), '
            '"protection_storage_masks": {"protection_storage_masks": [3]}'
            '}',
            str(payload)
        )

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        Register request payloads with the same data.
        """
        a = payloads.RegisterRequestPayload()
        b = payloads.RegisterRequestPayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.CERTIFICATE,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            ),
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )
        b = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.CERTIFICATE,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            ),
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_object_type(self):
        """
        Test that the equality operator returns False when comparing two
        Register request payloads with different object types.
        """
        a = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.SYMMETRIC_KEY
        )
        b = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.SECRET_DATA
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_template_attribute(self):
        """
        Test that the equality operator returns False when comparing two
        Register request payloads with different template attributes.
        """
        a = payloads.RegisterRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            value=enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            )
        )
        b = payloads.RegisterRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            value=enums.CryptographicUsageMask.SIGN.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_managed_object(self):
        """
        Test that the equality operator returns False when comparing two
        Register request payloads with different managed objects.
        """
        a = payloads.RegisterRequestPayload(
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            )
        )
        b = payloads.RegisterRequestPayload(
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.PGP,
                certificate_value=self.certificate_value
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_protection_storage_masks(self):
        """
        Test that the equality operator returns False when comparing two Create
        request payloads with different protection storage masks.
        """
        a = payloads.RegisterRequestPayload(
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )
        b = payloads.RegisterRequestPayload(
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.ON_SYSTEM.value |
                        enums.ProtectionStorageMask.OFF_SYSTEM.value
                    )
                ]
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        Register request payloads with different types.
        """
        a = payloads.RegisterRequestPayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        Register request payloads with the same data.
        """
        a = payloads.RegisterRequestPayload()
        b = payloads.RegisterRequestPayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.CERTIFICATE,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            ),
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )
        b = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.CERTIFICATE,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            enums.CryptographicUsageMask.SIGN.value |
                            enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            ),
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            ),
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_object_type(self):
        """
        Test that the inequality operator returns True when comparing two
        Register request payloads with different object types.
        """
        a = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.SYMMETRIC_KEY
        )
        b = payloads.RegisterRequestPayload(
            object_type=enums.ObjectType.SECRET_DATA
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_template_attribute(self):
        """
        Test that the inequality operator returns True when comparing two
        Register request payloads with different template attributes.
        """
        a = payloads.RegisterRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            value=enums.CryptographicUsageMask.VERIFY.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            )
        )
        b = payloads.RegisterRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "Cryptographic Usage Mask"
                        ),
                        attribute_value=primitives.Integer(
                            value=enums.CryptographicUsageMask.SIGN.value,
                            tag=enums.Tags.CRYPTOGRAPHIC_USAGE_MASK
                        )
                    )
                ]
            )
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_managed_object(self):
        """
        Test that the inequality operator returns True when comparing two
        Register request payloads with different managed objects.
        """
        a = payloads.RegisterRequestPayload(
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.X_509,
                certificate_value=self.certificate_value
            )
        )
        b = payloads.RegisterRequestPayload(
            managed_object=secrets.Certificate(
                certificate_type=enums.CertificateType.PGP,
                certificate_value=self.certificate_value
            )
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_protection_storage_masks(self):
        """
        Test that the inequality operator returns True when comparing two
        Register request payloads with different protection storage masks.
        """
        a = payloads.RegisterRequestPayload(
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.SOFTWARE.value |
                        enums.ProtectionStorageMask.HARDWARE.value
                    )
                ]
            )
        )
        b = payloads.RegisterRequestPayload(
            protection_storage_masks=objects.ProtectionStorageMasks(
                protection_storage_masks=[
                    (
                        enums.ProtectionStorageMask.ON_SYSTEM.value |
                        enums.ProtectionStorageMask.OFF_SYSTEM.value
                    )
                ]
            )
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        Register request payloads with different types.
        """
        a = payloads.RegisterRequestPayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)


class TestRegisterResponsePayload(testtools.TestCase):

    def setUp(self):
        super(TestRegisterResponsePayload, self).setUp()

        # Encoding obtained from the KMIP 1.1 testing document, Section 13.2.2.
        # Modified to include the template attribute.
        #
        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - 7091d0bf-548a-4d4a-93a6-6dd71cf75221
        #     Template Attribute
        #         Attribute
        #             Attribute Name - State
        #             Attribute Value - Pre-active
        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x60'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x37\x30\x39\x31\x64\x30\x62\x66\x2D\x35\x34\x38\x61\x2D\x34\x64'
            b'\x34\x61\x2D\x39\x33\x61\x36\x2D\x36\x64\x64\x37\x31\x63\x66\x37'
            b'\x35\x32\x32\x31\x00\x00\x00\x00'
            b'\x42\x00\x91\x01\x00\x00\x00\x28'
            b'\x42\x00\x08\x01\x00\x00\x00\x20'
            b'\x42\x00\x0A\x07\x00\x00\x00\x05'
            b'\x53\x74\x61\x74\x65\x00\x00\x00'
            b'\x42\x00\x0B\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
        )

        # Encoding obtained from the KMIP 1.1 testing document, Section 13.2.2.
        # Modified to include the template attribute.
        #
        # This encoding matches the following set of values:
        # Response Payload
        #     Template Attribute
        #         Attribute
        #             Attribute Name - State
        #             Attribute Value - Pre-active
        self.no_unique_identifier_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x30'
            b'\x42\x00\x91\x01\x00\x00\x00\x28'
            b'\x42\x00\x08\x01\x00\x00\x00\x20'
            b'\x42\x00\x0A\x07\x00\x00\x00\x05'
            b'\x53\x74\x61\x74\x65\x00\x00\x00'
            b'\x42\x00\x0B\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
        )

        # Encoding obtained from the KMIP 1.1 testing document, Section 13.2.2.
        #
        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - 7091d0bf-548a-4d4a-93a6-6dd71cf75221
        self.no_template_attribute_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x30'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x37\x30\x39\x31\x64\x30\x62\x66\x2D\x35\x34\x38\x61\x2D\x34\x64'
            b'\x34\x61\x2D\x39\x33\x61\x36\x2D\x36\x64\x64\x37\x31\x63\x66\x37'
            b'\x35\x32\x32\x31\x00\x00\x00\x00'
        )

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a Register response payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.RegisterResponsePayload,
            **kwargs
        )

        args = (payloads.RegisterResponsePayload(), 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_invalid_template_attribute(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the template attribute of a Register response payload.
        """
        kwargs = {'template_attribute': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Template attribute must be a TemplateAttribute structure.",
            payloads.RegisterResponsePayload,
            **kwargs
        )

        args = (
            payloads.RegisterResponsePayload(),
            'template_attribute',
            'invalid'
        )
        self.assertRaisesRegex(
            TypeError,
            "Template attribute must be a TemplateAttribute structure.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a Register response payload can be read from a data stream.
        """
        payload = payloads.RegisterResponsePayload()

        self.assertIsNone(payload.unique_identifier)
        self.assertIsNone(payload.template_attribute)

        payload.read(self.full_encoding)

        self.assertEqual(
            "7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            payload.unique_identifier
        )
        self.assertEqual(
            objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            ),
            payload.template_attribute
        )

    def test_read_kmip_2_0(self):
        """
        Test that a Register response payload can be read from a data stream
        encoded with the KMIP 2.0 format.
        """
        payload = payloads.RegisterResponsePayload()

        self.assertIsNone(payload.unique_identifier)
        self.assertIsNone(payload.template_attribute)

        payload.read(
            self.no_template_attribute_encoding,
            kmip_version=enums.KMIPVersion.KMIP_2_0
        )

        self.assertEqual(
            "7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            payload.unique_identifier
        )
        self.assertIsNone(payload.template_attribute)

    def test_read_missing_unique_identifier(self):
        """
        Test that an InvalidKmipEncoding error is raised during the decoding
        of a Register response payload when the unique identifier is missing
        from the encoding.
        """
        payload = payloads.RegisterResponsePayload()

        self.assertIsNone(payload.unique_identifier)
        self.assertIsNone(payload.template_attribute)

        args = (self.no_unique_identifier_encoding, )
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The Register response payload encoding is missing the unique "
            "identifier.",
            payload.read,
            *args
        )

    def test_read_missing_template_attribute(self):
        """
        Test that a Register response payload can be read from a data stream
        event when missing the template attribute.
        """
        payload = payloads.RegisterResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.template_attribute)

        payload.read(self.no_template_attribute_encoding)

        self.assertEqual(
            "7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            payload.unique_identifier
        )
        self.assertIsNone(payload.template_attribute)

    def test_write(self):
        """
        Test that a Register response payload can be written to a data stream.
        """
        payload = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )

        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_kmip_2_0(self):
        """
        Test that a Register response payload can be written to a data stream
        encoded with the KMIP 2.0 format.
        """
        payload = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )

        stream = utils.BytearrayStream()
        payload.write(stream, kmip_version=enums.KMIPVersion.KMIP_2_0)

        self.assertEqual(len(self.no_template_attribute_encoding), len(stream))
        self.assertEqual(str(self.no_template_attribute_encoding), str(stream))

    def test_write_missing_unique_identifier(self):
        """
        Test that an InvalidField error is raised during the encoding of a
        Register response payload when the payload is missing the unique
        identifier.
        """
        payload = payloads.RegisterResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )

        stream = utils.BytearrayStream()
        args = (stream, )
        self.assertRaisesRegex(
            exceptions.InvalidField,
            "The Register response payload is missing the unique identifier "
            "field.",
            payload.write,
            *args
        )

    def test_write_missing_template_attribute(self):
        """
        Test that a Register response payload can be written to a data stream
        even when missing the template attribute.
        """
        payload = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221"
        )

        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.no_template_attribute_encoding), len(stream))
        self.assertEqual(str(self.no_template_attribute_encoding), str(stream))

    def test_repr(self):
        """
        Test that repr can be applied to a Register response payload structure.
        """
        payload = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )
        self.assertEqual(
            "RegisterResponsePayload("
            "unique_identifier='7091d0bf-548a-4d4a-93a6-6dd71cf75221', "
            "template_attribute=Struct())",
            repr(payload)
        )

    def test_str(self):
        """
        Test that str can be applied to a Register response payload structure.
        """
        payload = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )
        self.assertEqual(
            '{'
            '"unique_identifier": "7091d0bf-548a-4d4a-93a6-6dd71cf75221", '
            '"template_attribute": Struct()'
            '}',
            str(payload)
        )

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        Register response payloads with the same data.
        """
        a = payloads.RegisterResponsePayload()
        b = payloads.RegisterResponsePayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )
        b = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        Register response payloads with different unique identifiers.
        """
        a = payloads.RegisterResponsePayload(unique_identifier="a")
        b = payloads.RegisterResponsePayload(unique_identifier="b")

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_template_attribute(self):
        """
        Test that the equality operator returns False when comparing two
        Register response payloads with different template attributes.
        """
        a = payloads.RegisterResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )
        b = payloads.RegisterResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        Register response payloads with different types.
        """
        a = payloads.RegisterResponsePayload()
        b = "invalid"

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        Register response payloads with the same data.
        """
        a = payloads.RegisterResponsePayload()
        b = payloads.RegisterResponsePayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )
        b = payloads.RegisterResponsePayload(
            unique_identifier="7091d0bf-548a-4d4a-93a6-6dd71cf75221",
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        Register response payloads with different unique identifiers.
        """
        a = payloads.RegisterResponsePayload(unique_identifier="a")
        b = payloads.RegisterResponsePayload(unique_identifier="b")

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_template_attribute(self):
        """
        Test that the inequality operator returns True when comparing two
        Register response payloads with different template attributes.
        """
        a = payloads.RegisterResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.PRE_ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )
        b = payloads.RegisterResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            "State"
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.State,
                            value=enums.State.ACTIVE,
                            tag=enums.Tags.STATE
                        )
                    )
                ]
            )
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        Register response payloads with different types.
        """
        a = payloads.RegisterResponsePayload()
        b = "invalid"

        self.assertTrue(a != b)
        self.assertTrue(b != a)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools import TestCase

from kmip.core.enums import ResultStatus
from kmip.core.enums import ResultReason

from kmip.pie.exceptions import ClientConnectionFailure
from kmip.pie.exceptions import ClientConnectionNotOpen
from kmip.pie.exceptions import KmipOperationFailure


class TestClientConnectionFailure(TestCase):
    """
    Test suite for ClientConnectionFailure.
    """

    def setUp(self):
        super(TestClientConnectionFailure, self).setUp()

    def tearDown(self):
        super(TestClientConnectionFailure, self).tearDown()

    def test_init(self):
        """
        Test that a ClientConnectionFailure exception can be instantiated.
        """
        exc = ClientConnectionFailure()
        self.assertIsInstance(exc, Exception)

    def test_message(self):
        """
        Test that a ClientConnectionFailure exception message can be set
        properly.
        """
        exc = ClientConnectionFailure("test message")
        self.assertEqual("test message", str(exc))


class TestClientConnectionNotOpen(TestCase):
    """
    Test suite for ClientConnectionNotOpen.
    """

    def setUp(self):
        super(TestClientConnectionNotOpen, self).setUp()

    def tearDown(self):
        super(TestClientConnectionNotOpen, self).tearDown()

    def test_init(self):
        """
        Test that a ClientConnectionNotOpen exception can be instantiated.
        """
        exc = ClientConnectionNotOpen()
        self.assertIsInstance(exc, Exception)
        self.assertEqual("client connection not open", str(exc))


class TestKmipOperationFailure(TestCase):
    """
    Test suite for KmipOperationFailure.
    """

    def setUp(self):
        super(TestKmipOperationFailure, self).setUp()

    def tearDown(self):
        super(TestKmipOperationFailure, self).tearDown()

    def test_init(self):
        """
        Test that a KmipOperationFailure exception can be instantiated.
        """
        exc = KmipOperationFailure(
            ResultStatus.OPERATION_FAILED,
            ResultReason.GENERAL_FAILURE,
            "Test error message.")
        self.assertIsInstance(exc, Exception)

    def test_message(self):
        """
        Test that a KmipOperationFailure exception message and attributes can
        be set properly.
        """
        status = ResultStatus.OPERATION_FAILED
        reason = ResultReason.GENERAL_FAILURE
        exc = KmipOperationFailure(status, reason, "Test error message.")

        msg = "{0}: {1} - {2}".format(
            status.name, reason.name, "Test error message.")

        self.assertEqual(msg, str(exc))
        self.assertEqual(status, exc.status)
        self.assertEqual(reason, exc.reason)
        self.assertEqual("Test error message.", exc.message)



# Copyright (c) 2016 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import enums
from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship

import sqlalchemy.types as types


Base = declarative_base()


def attribute_append_factory(index_attribute):
    def attribute_append(list_container, list_attribute, initiator):
        index = getattr(list_container, index_attribute)
        list_attribute.index = index
        setattr(list_container, index_attribute, index + 1)
        return list_attribute
    return attribute_append


class UsageMaskType(types.TypeDecorator):
    """
    Converts a list of enums.CryptographicUsageMask Enums in an integer
    bitmask. This allows the database to only store an integer instead of a
    list of enumbs. This also does the reverse of converting an integer bit
    mask into a list enums.CryptographicUsageMask Enums.
    """

    impl = types.Integer

    def process_bind_param(self, value, dialect):
        """
        Returns the integer value of the usage mask bitmask. This value is
        stored in the database.

        Args:
            value(list<enums.CryptographicUsageMask>): list of enums in the
            usage mask
            dialect(string): SQL dialect
        """
        bitmask = 0x00
        for e in value:
            bitmask = bitmask | e.value
        return bitmask

    def process_result_value(self, value, dialect):
        """
        Returns a new list of enums.CryptographicUsageMask Enums. This converts
        the integer value into the list of enums.

        Args:
            value(int): The integer value stored in the database that is used
                to create the list of enums.CryptographicUsageMask Enums.
            dialect(string): SQL dialect
        """
        masks = list()
        if value:
            for e in enums.CryptographicUsageMask:
                if e.value & value:
                    masks.append(e)
        return masks


class EnumType(types.TypeDecorator):
    """
    Converts a Python enum to an integer before storing it in the database.
    This also does the reverse of converting an integer into an enum object.
    This allows enums to be stored in a database.
    """

    impl = types.Integer

    def __init__(self, cls):
        """
        Create a new EnumType. This new EnumType requires a class object in the
        constructor. The class is used to construct new instances of the Enum
        when the integer value is retrieved from the database.

        Args:
            cls(class): An Enum class used to create new instances from integer
                values.
        """
        super(EnumType, self).__init__()
        self._cls = cls

    def process_bind_param(self, value, dialect):
        """
        Returns the integer value of the Enum. This value is stored in the
        database.

        Args:
            value(Enum): An Enum instance whose integer value is to be stored.
            dialect(string): SQL dialect
        """
        if value:
            return value.value
        else:
            return -1

    def process_result_value(self, value, dialect):
        """
        Returns a new Enum representing the value stored in the database. The
        Enum class type of the returned object is that of the cls parameter in
        the __init__ call.

        Args:
            value(int): The integer value stored in the database that is used
                to create the Enum
            dialect(string): SQL dialect
        """
        if value == -1:
            return None
        return self._cls(value)


class ManagedObjectName(Base):

    __tablename__ = 'managed_object_names'
    id = Column('id', Integer, primary_key=True)
    mo_uid = Column('mo_uid', Integer, ForeignKey('managed_objects.uid'))
    name = Column('name', String)
    index = Column('name_index', Integer)
    name_type = Column('name_type', EnumType(enums.NameType))

    mo = relationship('ManagedObject', back_populates='_names')

    def __init__(self, name, index=0,
                 name_type=enums.NameType.UNINTERPRETED_TEXT_STRING):
        self.name = name
        self.index = index
        self.name_type = name_type

    def __repr__(self):
        return ("<ManagedObjectName(name='%s', index='%d', type='%s')>" %
                (self.name, self.index, self.name_type))

    def __eq__(self, other):
        if isinstance(other, ManagedObjectName):
            if self.name != other.name:
                return False
            elif self.index != other.index:
                return False
            elif self.name_type != other.name_type:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, ManagedObjectName):
            return not (self == other)
        else:
            return NotImplemented
